@@ -25,11 +25,15 @@
 #include "document/document_glom.h"
 #include "data_structure/glomconversions.h"
 #include "mode_data/dialog_choose_field.h"
+#include "layout_item_dialogs/dialog_field_layout.h"
+#include "layout_item_dialogs/dialog_notebook.h"
+#include "mode_design/dialog_textobject.h"
 //#include "dialog_layout_report.h"
 #include "utils.h"
 #include "data_structure/glomconversions.h"
 #include "data_structure/layout/report_parts/layoutitem_summary.h"
 #include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include "data_structure/layout/report_parts/layoutitem_verticalgroup.h"
 #include "python_embed/glom_python.h"
 #include <glibmm/i18n.h>
 //#include <libgnomeui/gnome-app-helper.h>
@@ -1269,12 +1273,12 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
   return insert_succeeded;
 }
 
-sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const Glib::ustring& table_name)
+sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const Glib::ustring& table_name, Gtk::Window* transient_for)
 {
-  return offer_field_list(sharedptr<LayoutItem_Field>(), table_name);
+  return offer_field_list(sharedptr<LayoutItem_Field>(), table_name, transient_for);
 }
 
-sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const sharedptr<LayoutItem_Field>& start_field, const Glib::ustring& table_name)
+sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for)
 {
   sharedptr<LayoutItem_Field> result;
 
@@ -1287,9 +1291,12 @@ sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const sharedptr<LayoutItem
 
     if(dialog)
     {
+      if(transient_for)
+        dialog->set_transient_for(*transient_for);
+
       dialog->set_document(get_document(), table_name, start_field);
       //TODO: dialog->set_transient_for(*get_app_window());
-      int response = dialog->run();
+      const int response = dialog->run();
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen field:
@@ -1307,91 +1314,237 @@ sharedptr<LayoutItem_Field> Base_DB::offer_field_list(const sharedptr<LayoutItem
   return result;
 }
 
-void Base_DB::fill_full_field_details(const Glib::ustring& parent_table_name, sharedptr<LayoutItem_Field>& layout_item)
+sharedptr<LayoutItem_Field> Base_DB::offer_field_formatting(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for)
 {
-  Glib::ustring table_name = parent_table_name;
+  sharedptr<LayoutItem_Field> result;
 
-  //It might be a related field:
-  if(layout_item->get_has_relationship_name())
+  try
   {
-    sharedptr<const Relationship> rel = layout_item->get_relationship();;
-    table_name = rel->get_to_table();
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
+
+    Dialog_FieldLayout* dialog = 0;
+    refXml->get_widget_derived("dialog_layout_field_properties", dialog);
+
+    if(dialog)
+    {
+      if(transient_for)
+        dialog->set_transient_for(*transient_for);
+
+      add_view(dialog);
+
+      dialog->set_field(start_field, table_name);
+
+
+      const int response = dialog->run();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen field:
+        result = dialog->get_field_chosen();
+      }
+
+      remove_view(dialog);
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
   }
 
-  layout_item->set_full_field_details( get_document()->get_field(table_name, layout_item->get_name()) );
+  return result;
 }
 
-Glib::ustring Base_DB::get_layout_item_table_name(const sharedptr<const LayoutItem_Field>& layout_item, const Glib::ustring& table_name)
+sharedptr<LayoutItem_Text> Base_DB::offer_textobject(const sharedptr<LayoutItem_Text>& start_textobject, Gtk::Window* transient_for)
 {
-  if(!layout_item->get_has_relationship_name())
-    return table_name;
-  else
-  {
-    const Glib::ustring relationship_name = layout_item->get_relationship_name();
+  sharedptr<LayoutItem_Text> result;
 
-    //TODO: We should not need to do this. It should be updated in the LayoutItem_Field already.
-    Document_Glom* document = get_document();
-    sharedptr<Relationship> relationship = document->get_relationship(table_name, relationship_name);
-    if(relationship)
-      return relationship->get_to_table();
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_textobject");
+
+    Dialog_TextObject* dialog = 0;
+    refXml->get_widget_derived("window_textobject", dialog);
+    if(dialog)
+    {
+      if(transient_for)
+        dialog->set_transient_for(*transient_for);
+
+      dialog->set_textobject(start_textobject, Glib::ustring());
+      //dialog->set_transient_for(*this);
+      const int response = dialog->run();
+      dialog->hide();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen relationship:
+        result = dialog->get_textobject();
+      }
+
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
   }
 
-  return Glib::ustring();
+  return result;
+}
+
+sharedptr<LayoutItem_Notebook> Base_DB::offer_notebook(const sharedptr<LayoutItem_Notebook>& start_notebook, Gtk::Window* transient_for)
+{
+  sharedptr<LayoutItem_Notebook> result;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_notebook");
+
+    Dialog_Notebook* dialog = 0;
+    refXml->get_widget_derived("dialog_notebook", dialog);
+    if(dialog)
+    {
+      if(transient_for)
+        dialog->set_transient_for(*transient_for);
+
+      dialog->set_notebook(start_notebook);
+      //dialog->set_transient_for(*this);
+      const int response = dialog->run();
+      dialog->hide();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen relationship:
+        result = dialog->get_notebook();
+      }
+
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
+void Base_DB::fill_full_field_details(const Glib::ustring& parent_table_name, sharedptr<LayoutItem_Field>& layout_item)
+{
+  const Glib::ustring table_name = layout_item->get_table_used(parent_table_name);
+
+  layout_item->set_full_field_details( get_document()->get_field(table_name, layout_item->get_name()) );
 }
 
 void Base_DB::report_build_summary(const Glib::ustring& table_name, xmlpp::Element& parent_node, const sharedptr<LayoutItem_Summary>& summary, const Glib::ustring& where_clause)
 {
   //Add XML node:
-  xmlpp::Element* nodeSummary = parent_node.add_child("summary");
+  xmlpp::Element* node = parent_node.add_child("summary");
 
   //Get fields
-  GlomUtils::type_vecLayoutFields fieldsToGet;
+  type_vecLayoutItems itemsToGet;
   for(LayoutGroup::type_map_items::iterator iterChildren = summary->m_map_items.begin(); iterChildren != summary->m_map_items.end(); ++iterChildren)
   {
     sharedptr<LayoutItem> item = iterChildren->second;
-    sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(item);
-    if(pField)
+
+    sharedptr<LayoutItem_GroupBy> pGroupBy = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
+    if(pGroupBy)
     {
-      fill_full_field_details(table_name, pField);
-      fieldsToGet.push_back( glom_sharedptr_clone(pField) );
+      //Recurse, adding a sub-groupby block:
+      report_build_groupby(table_name, *node, pGroupBy, where_clause);
     }
     else
     {
-      sharedptr<LayoutItem_GroupBy> pGroupBy = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
-      if(pGroupBy)
+      sharedptr<LayoutItem_Summary> pSummary = sharedptr<LayoutItem_Summary>::cast_dynamic(item);
+      if(pSummary)
       {
-        //Recurse, adding a sub-groupby block:
-        report_build_groupby(table_name, *nodeSummary, pGroupBy, where_clause);
+        //Recurse, adding a summary block:
+        report_build_summary(table_name, *node, pSummary, where_clause);
       }
       else
       {
-        sharedptr<LayoutItem_Summary> pSummary = sharedptr<LayoutItem_Summary>::cast_dynamic(item);
-        if(pSummary)
-        {
-          //Recurse, adding a summary block:
-          report_build_summary(table_name, *nodeSummary, pSummary, where_clause);
-        }
+        itemsToGet.push_back( glom_sharedptr_clone(item) );
       }
     }
   }
 
-  if(!fieldsToGet.empty())
+  if(!itemsToGet.empty())
   {
     //Rows, with data:
-    report_build_records(table_name, *nodeSummary, fieldsToGet, where_clause, Glib::ustring() /* No sort_clause because there is only one row */);
+    report_build_records(table_name, *node, itemsToGet, where_clause, Glib::ustring() /* No sort_clause because there is only one row */);
+  }
+}
+
+
+
+void Base_DB::report_build_groupby_children(const Glib::ustring& table_name, xmlpp::Element& node, const sharedptr<LayoutItem_GroupBy>& group_by, const Glib::ustring& where_clause)
+{
+  //Get data and add child rows:
+  type_vecLayoutItems itemsToGet;
+  for(LayoutGroup::type_map_items::iterator iterChildren = group_by->m_map_items.begin(); iterChildren != group_by->m_map_items.end(); ++iterChildren)
+  {
+    sharedptr<LayoutItem> item = iterChildren->second;
+
+    sharedptr<LayoutItem_GroupBy> pGroupBy = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
+    if(pGroupBy)
+    {
+      //Recurse, adding a sub-groupby block:
+      report_build_groupby(table_name, node, pGroupBy, where_clause);
+    }
+    else
+    {
+      sharedptr<LayoutItem_Summary> pSummary = sharedptr<LayoutItem_Summary>::cast_dynamic(item);
+      if(pSummary)
+      {
+        //Recurse, adding a summary block:
+        report_build_summary(table_name, node, pSummary, where_clause);
+      }
+      else
+      {
+        itemsToGet.push_back( glom_sharedptr_clone(item) );
+      }
+    }
+  }
+
+  if(!itemsToGet.empty())
+  {
+    //Rows, with data:
+
+    //Sort Fields:
+    Glib::ustring sort_clause;
+    {
+      if(group_by->get_has_fields_sort_by())
+      {
+        sort_clause += " ORDER BY ";
+
+        bool first = true;
+        LayoutItem_GroupBy::type_list_sort_fields group_sort_fields = group_by->get_fields_sort_by();
+        for(LayoutItem_GroupBy::type_list_sort_fields::const_iterator iter = group_sort_fields.begin(); iter != group_sort_fields.end(); ++iter)
+        {
+          if(!first)
+            sort_clause += ", ";
+
+          sharedptr<LayoutItem_Field> item_field = sharedptr<LayoutItem_Field>::cast_dynamic(iter->first);
+          if(item_field)
+          {
+            sort_clause += "\"" + item_field->get_sql_table_or_join_alias_name(table_name) + "\".\"" + item_field->get_name() + "\" " + (iter->second ? "ASC" : "DESC");
+            first = false;
+          }
+        }
+      }
+    }
+
+    report_build_records(table_name, node, itemsToGet, where_clause, sort_clause);
   }
 }
 
 void Base_DB::report_build_groupby(const Glib::ustring& table_name, xmlpp::Element& parent_node, const sharedptr<LayoutItem_GroupBy>& group_by, const Glib::ustring& where_clause_parent)
 {
   //Get the possible heading values.
-  sharedptr<LayoutItem_Field> field_group_by = group_by->get_field_group_by();
-  if(field_group_by)
+  if(group_by->get_has_field_group_by())
   {
+    sharedptr<LayoutItem_Field> field_group_by = group_by->get_field_group_by();
     fill_full_field_details(table_name, field_group_by);
 
     //Get the possible group values, ignoring repeats by using GROUP BY.
-    const Glib::ustring group_field_table_name = get_layout_item_table_name(field_group_by, table_name);
+    const Glib::ustring group_field_table_name = field_group_by->get_table_used(table_name);
     Glib::ustring sql_query = "SELECT \"" + group_field_table_name + "\".\"" + field_group_by->get_name() + "\""
       " FROM \"" + group_field_table_name + "\"";
 
@@ -1410,6 +1563,7 @@ void Base_DB::report_build_groupby(const Glib::ustring& table_name, xmlpp::Eleme
 
         //Add XML node:
         xmlpp::Element* nodeGroupBy = parent_node.add_child("group_by");
+        Document_Glom::set_node_attribute_value_as_decimal_double(nodeGroupBy, "border_width", group_by->get_border_width());
 
         nodeGroupBy->set_attribute("group_field", field_group_by->get_title_or_name());
         nodeGroupBy->set_attribute("group_value",
@@ -1424,89 +1578,92 @@ void Base_DB::report_build_groupby(const Glib::ustring& table_name, xmlpp::Eleme
         {
           xmlpp::Element* nodeSecondaryFields = nodeGroupBy->add_child("secondary_fields");
 
-          GlomUtils::type_vecLayoutFields fieldsToGet;
+          type_vecLayoutItems itemsToGet;
           for(LayoutGroup::type_map_items::iterator iterChildren = group_by->m_group_secondary_fields->m_map_items.begin(); iterChildren != group_by->m_group_secondary_fields->m_map_items.end(); ++iterChildren)
           {
             sharedptr<LayoutItem> item = iterChildren->second;
-            sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(item);
-            if(pField)
-            {
-              fill_full_field_details(table_name, pField);
-              fieldsToGet.push_back( glom_sharedptr_clone(pField) );
-            }
+            itemsToGet.push_back( glom_sharedptr_clone(item) );
           }
 
-          if(!fieldsToGet.empty())
+          if(!itemsToGet.empty())
           {
-            report_build_records(table_name, *nodeSecondaryFields, fieldsToGet, where_clause, Glib::ustring(), true /* one record only */);
+            report_build_records(table_name, *nodeSecondaryFields, itemsToGet, where_clause, Glib::ustring(), true /* one record only */);
           }
         }
 
-
         //Get data and add child rows:
-        GlomUtils::type_vecLayoutFields fieldsToGet;
-        for(LayoutGroup::type_map_items::iterator iterChildren = group_by->m_map_items.begin(); iterChildren != group_by->m_map_items.end(); ++iterChildren)
-        {
-          sharedptr<LayoutItem> item = iterChildren->second;
-          sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(item);
-          if(pField)
-          {
-            fill_full_field_details(table_name, pField);
-            fieldsToGet.push_back( glom_sharedptr_clone(pField) );
-          }
-          else
-          {
-            sharedptr<LayoutItem_GroupBy> pGroupBy = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
-            if(pGroupBy)
-            {
-              //Recurse, adding a sub-groupby block:
-              report_build_groupby(table_name, *nodeGroupBy, pGroupBy, where_clause);
-            }
-            else
-            {
-              sharedptr<LayoutItem_Summary> pSummary = sharedptr<LayoutItem_Summary>::cast_dynamic(item);
-              if(pSummary)
-              {
-                //Recurse, adding a summary block:
-                report_build_summary(table_name, *nodeGroupBy, pSummary, where_clause);
-              }
-            }
-          }
-        }
-
-        if(!fieldsToGet.empty())
-        {
-          //Rows, with data:
-          Glib::ustring sort_clause;
-          if(group_by->get_has_field_sort_by())
-            sort_clause = group_by->get_field_sort_by()->get_name(); //TODO: Deal with related fields too.
-
-          report_build_records(table_name, *nodeGroupBy, fieldsToGet, where_clause, sort_clause);
-        }
+        report_build_groupby_children(table_name, *nodeGroupBy, group_by, where_clause);
       }
     }
   }
+  else
+  {
+    //There is no group-by field, so ouput all the found records.
+    //For instance, the user could use the GroupBy part just to specify a sort, though that would be a bit of a hack:
+    xmlpp::Element* nodeGroupBy = parent_node.add_child("group_by"); //We need this to create the HTML table.
+    Document_Glom::set_node_attribute_value_as_decimal_double(nodeGroupBy, "border_width", group_by->get_border_width());
+    report_build_groupby_children(table_name, *nodeGroupBy, group_by, where_clause_parent);
+  }
+}
+
+void Base_DB::report_build_records_get_fields(const Glib::ustring& table_name, const sharedptr<LayoutGroup>& group, type_vecLayoutFields& items)
+{
+  for(LayoutGroup::type_map_items::iterator iterChildren = group->m_map_items.begin(); iterChildren != group->m_map_items.end(); ++iterChildren)
+  {
+    sharedptr<LayoutItem> item = iterChildren->second;
+
+    sharedptr<LayoutItem_VerticalGroup> pVerticalGroup = sharedptr<LayoutItem_VerticalGroup>::cast_dynamic(item);
+    if(pVerticalGroup)
+    {
+      report_build_records_get_fields(table_name, pVerticalGroup, items);
+    }
+    else
+    {
+      sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+      if(pField)
+        items.push_back(pField);
+    }
+  }
 }
 
-void Base_DB::report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const GlomUtils::type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& sort_clause, bool one_record_only)
+void Base_DB::report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const type_vecLayoutItems& items, const Glib::ustring& where_clause, const Glib::ustring& sort_clause, bool one_record_only)
 {
-  if(!fieldsToGet.empty())
+  if(!items.empty())
   {
-    //Field headings:
-    for(GlomUtils::type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+    //Add Field headings:
+    for(type_vecLayoutItems::const_iterator iter = items.begin(); iter != items.end(); ++iter)
     {
-      sharedptr<LayoutItem_Field> layout_item = *iter;
+      sharedptr<LayoutItem> layout_item = *iter;
+      sharedptr<LayoutItem_Field> layoutitem_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
 
-      xmlpp::Element* nodeFieldHeading = 0;
-      if(layout_item->get_glom_type() == Field::TYPE_NUMERIC)
-        nodeFieldHeading = parent_node.add_child("field_heading_numeric"); //This is quite hacky. murrayc.
-      else
-        nodeFieldHeading = parent_node.add_child("field_heading");
+      //This adds a field heading (and therefore, column) for fields, or for a vertical group. 
+      xmlpp::Element* nodeFieldHeading = parent_node.add_child("field_heading");
+      if(layoutitem_field && layoutitem_field->get_glom_type() == Field::TYPE_NUMERIC)
+        nodeFieldHeading->set_attribute("field_type", "numeric"); //TODO: More sophisticated formatting.
 
       nodeFieldHeading->set_attribute("name", layout_item->get_name()); //Not really necessary, but maybe useful.
       nodeFieldHeading->set_attribute("title", layout_item->get_title_or_name());
     }
 
+    //Get list of fields to get from the database.
+    GlomUtils::type_vecLayoutFields fieldsToGet;
+    for(type_vecLayoutItems::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+    {
+      sharedptr<LayoutItem> layout_item = *iter;
+      sharedptr<LayoutItem_Field> layoutitem_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
+      if(layoutitem_field)
+        fieldsToGet.push_back(layoutitem_field);
+      else
+      {
+        sharedptr<LayoutItem_VerticalGroup> vertical_group = sharedptr<LayoutItem_VerticalGroup>::cast_dynamic(layout_item);
+        if(vertical_group)
+        {
+          //Get all the fields in this group:
+          report_build_records_get_fields(table_name, vertical_group, fieldsToGet);
+        }
+      }
+    }
+
     Glib::ustring sql_query = GlomUtils::build_sql_select_with_where_clause(table_name,
       fieldsToGet,
       where_clause, sort_clause);
@@ -1527,30 +1684,37 @@ void Base_DB::report_build_records(const Glib::ustring& table_name, xmlpp::Eleme
       {
         xmlpp::Element* nodeRow = parent_node.add_child("row");
 
-        for(guint col = 0; col < fieldsToGet.size(); ++col)
+        guint colField = 0;
+        for(type_vecLayoutItems::const_iterator iter = items.begin(); iter != items.end(); ++iter)
         {
-          sharedptr<LayoutItem_Field> field = fieldsToGet[col];
-          const Field::glom_field_type field_type = field->get_glom_type();
-
           xmlpp::Element* nodeField = 0;
-          if(field_type == Field::TYPE_NUMERIC)
-             nodeField = nodeRow->add_child("field_numeric"); //TODO: I would prefer just to add a field_type attribute to the "field" node and use an <xsl::if> instead. murrayc.
-          else
-             nodeField = nodeRow->add_child("field");
 
-          nodeField->set_attribute("name", field->get_name()); //Not really necessary, but maybe useful.
-
-          Glib::ustring text_value = GlomConversions::get_text_for_gda_value(field_type, datamodel->get_value_at(col, row), field->get_formatting_used().m_numeric_format);
-
-          //The Postgres summary functions return NULL when summarising NULL records, but 0 is more sensible:
-          if(text_value.empty() && sharedptr<LayoutItem_FieldSummary>::cast_dynamic(field) && (field_type == Field::TYPE_NUMERIC))
+          sharedptr<LayoutItem> item = *iter;
+          sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+          if(field)
           {
-            //Use get_text_for_gda_value() instead of "0" so we get the correct numerical formatting:
-            Gnome::Gda::Value value = GlomConversions::parse_value(0);
-            text_value = GlomConversions::get_text_for_gda_value(field_type, value, field->get_formatting_used().m_numeric_format);
+            report_build_records_field(table_name, *nodeRow, field, datamodel, row, colField);
+          }
+          else
+          {
+            sharedptr<LayoutItem_Text> item_text = sharedptr<LayoutItem_Text>::cast_dynamic(item);
+            if(item_text)
+            {
+              report_build_records_text(table_name, *nodeRow, item_text);
+            }
+            else
+            {
+              sharedptr<LayoutItem_VerticalGroup> item_verticalgroup = sharedptr<LayoutItem_VerticalGroup>::cast_dynamic(item);
+              if(item_verticalgroup)
+              {
+                report_build_records_vertical_group(table_name, *nodeRow, item_verticalgroup, datamodel, row, colField);
+                //TODO
+              }
+            }
           }
 
-          nodeField->set_attribute("value", text_value);
+          if(nodeField)
+            nodeField->set_attribute("name", item->get_name()); //Not really necessary, but maybe useful.
         }
       }
 
@@ -1561,6 +1725,61 @@ void Base_DB::report_build_records(const Glib::ustring& table_name, xmlpp::Eleme
   }
 }
 
+void Base_DB::report_build_records_field(const Glib::ustring& table_name, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Field>& field, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& colField, bool vertical)
+{
+  const Field::glom_field_type field_type = field->get_glom_type();
+
+  xmlpp::Element* nodeField = nodeParent.add_child(vertical ? "field_vertical" : "field");
+  if(field_type == Field::TYPE_NUMERIC)
+    nodeField->set_attribute("field_type", "numeric"); //TODO: More sophisticated formatting.
+
+  Glib::ustring text_value = GlomConversions::get_text_for_gda_value(field_type, datamodel->get_value_at(colField, row), field->get_formatting_used().m_numeric_format);
+
+  //The Postgres summary functions return NULL when summarising NULL records, but 0 is more sensible:
+  if(text_value.empty() && sharedptr<const LayoutItem_FieldSummary>::cast_dynamic(field) && (field_type == Field::TYPE_NUMERIC))
+  {
+    //Use get_text_for_gda_value() instead of "0" so we get the correct numerical formatting:
+    Gnome::Gda::Value value = GlomConversions::parse_value(0);
+    text_value = GlomConversions::get_text_for_gda_value(field_type, value, field->get_formatting_used().m_numeric_format);
+  }
+
+  nodeField->set_attribute("title", field->get_title_or_name()); //Not always used, but useful.
+  nodeField->set_attribute("value", text_value);
+
+  ++colField;
+}
+
+void Base_DB::report_build_records_text(const Glib::ustring& table_name, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Text>& textobject)
+{
+  //Text object:
+  xmlpp::Element* nodeField = nodeParent.add_child("field"); //We reuse this node type for text objects.
+  nodeField->set_attribute("value", textobject->get_text());
+}
+
+void Base_DB::report_build_records_vertical_group(const Glib::ustring& table_name, xmlpp::Element& parentNode, const sharedptr<LayoutItem_VerticalGroup>& group, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& field_index)
+{
+  xmlpp::Element* nodeGroupVertical = parentNode.add_child("group_vertical");
+
+  for(LayoutGroup::type_map_items::iterator iterChildren = group->m_map_items.begin(); iterChildren != group->m_map_items.end(); ++iterChildren)
+  {
+    sharedptr<LayoutItem> item = iterChildren->second;
+
+    sharedptr<LayoutItem_VerticalGroup> pVerticalGroup = sharedptr<LayoutItem_VerticalGroup>::cast_dynamic(item);
+    if(pVerticalGroup)
+    {
+      report_build_records_vertical_group(table_name, *nodeGroupVertical, pVerticalGroup, datamodel, row, field_index);
+    }
+    else
+    {
+      sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+      if(pField)
+      {
+        report_build_records_field(table_name, *nodeGroupVertical, pField, datamodel, row, field_index, true /* vertical, so we get a row for each field too. */);
+      }
+    }
+  }
+}
+
 
 void Base_DB::report_build(const Glib::ustring& table_name, const sharedptr<const Report>& report, const Glib::ustring& where_clause)
 {
@@ -1588,7 +1807,7 @@ void Base_DB::report_build(const Glib::ustring& table_name, const sharedptr<cons
 
   nodeRoot->set_attribute("title", report->get_title_or_name());
 
-  GlomUtils::type_vecLayoutFields fieldsToGet_TopLevel;
+  type_vecLayoutItems itemsToGet_TopLevel;
 
   for(LayoutGroup::type_map_items::const_iterator iter = report->m_layout_group->m_map_items.begin(); iter != report->m_layout_group->m_map_items.end(); ++iter)
   {
@@ -1608,20 +1827,16 @@ void Base_DB::report_build(const Glib::ustring& table_name, const sharedptr<cons
       }
       else
       {
-        sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(pPart);
-        if(pField)
-        {
-          fieldsToGet_TopLevel.push_back( glom_sharedptr_clone(pField) );
-        }
+        itemsToGet_TopLevel.push_back( glom_sharedptr_clone(pPart) );
       }
     }
   }
 
   //Add top-level records, outside of any groupby or summary, if fields have been specified:
-  if(!fieldsToGet_TopLevel.empty())
+  if(!itemsToGet_TopLevel.empty())
   {
     xmlpp::Element* nodeGroupBy = nodeParent->add_child("ungrouped_records");
-    report_build_records(table_name, *nodeGroupBy, fieldsToGet_TopLevel, where_clause, Glib::ustring() /* no sort clause */);
+    report_build_records(table_name, *nodeGroupBy, itemsToGet_TopLevel, where_clause, Glib::ustring() /* no sort clause */);
   }
 
   GlomUtils::transform_and_open(*pDocument, "print_report_to_html.xsl");
@@ -1772,8 +1987,6 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
 {
   //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): table_name=%s, all_db_fields.size()=%d, group->name=%s", table_name.c_str(), all_db_fields.size(), group->get_name().c_str());
 
-  const Document_Glom* document = get_document();
-
   LayoutGroup::type_map_const_items items = group->get_items();
   for(LayoutGroup::type_map_const_items::const_iterator iterItems = items.begin(); iterItems != items.end(); ++iterItems)
   {
@@ -1787,34 +2000,24 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
 
       if(item_field->get_has_relationship_name()) //If it's a field in a related table.
       {
-        //Get the full field information:
-        const Glib::ustring relationship_name = item_field->get_relationship_name();
-        sharedptr<Relationship> relationship = document->get_relationship(table_name, relationship_name);
-        if(relationship)
+        //TODO_Performance: get_fields_for_table_one_field() is probably very inefficient
+        sharedptr<Field> field = get_fields_for_table_one_field(item_field->get_table_used(table_name), item->get_name());
+        if(field)
         {
-          //TODO_Performance: get_fields_for_table_one_field() is probably very inefficient
-          sharedptr<Field> field = get_fields_for_table_one_field(relationship->get_to_table(), item->get_name());
-          if(field)
-          {
-            sharedptr<LayoutItem_Field> layout_item = glom_sharedptr_clone(item_field); //TODO_Performance: Reduce the copying.
-            layout_item->set_full_field_details(field); //Fill in the full field information for later.
+          sharedptr<LayoutItem_Field> layout_item = glom_sharedptr_clone(item_field); //TODO_Performance: Reduce the copying.
+          layout_item->set_full_field_details(field); //Fill in the full field information for later.
 
 
-            //TODO_Performance: We do this once for each related field, even if there are 2 from the same table:
-            const Privileges privs_related = get_current_privs(relationship->get_to_table());
-            layout_item->m_priv_view = privs_related.m_view;
-            layout_item->m_priv_edit = privs_related.m_edit;
+          //TODO_Performance: We do this once for each related field, even if there are 2 from the same table:
+          const Privileges privs_related = get_current_privs(item_field->get_table_used(table_name));
+          layout_item->m_priv_view = privs_related.m_view;
+          layout_item->m_priv_edit = privs_related.m_edit;
 
-            vecFields.push_back(layout_item);
-          }
-          else
-          {
-            std::cerr << "Base_DB::get_table_fields_to_show_for_sequence_add_group(): related field not found: relationship=" << relationship_name << ", field=" << item->get_name() << std::endl;
-          }
+          vecFields.push_back(layout_item);
         }
         else
         {
-          std::cerr << "Base_DB::get_table_fields_to_show_for_sequence_add_group(): related field's relationship not found: parent_table_name=" << table_name << ", relationship_name=" << relationship_name << std::endl;
+          std::cerr << "Base_DB::get_table_fields_to_show_for_sequence_add_group(): related field not found: field=" << item->get_layout_display_name() << std::endl;
         }
       }
       else //It's a regular field in the table:
