@@ -24,13 +24,15 @@
 #include <glibmm/i18n.h>
 
 LayoutGroup::LayoutGroup()
-: m_columns_count(1) //A sensible default
+: m_columns_count(1), //A sensible default
+  m_border_width(0)
 {
 }
 
 LayoutGroup::LayoutGroup(const LayoutGroup& src)
 : LayoutItem(src),
-  m_columns_count(src.m_columns_count)
+  m_columns_count(src.m_columns_count),
+  m_border_width(src.m_border_width)
 {
   //Deep copy of the items map:
   for(type_map_items::const_iterator iter = src.m_map_items.begin(); iter != src.m_map_items.end(); ++iter)
@@ -65,6 +67,8 @@ LayoutGroup& LayoutGroup::operator=(const LayoutGroup& src)
 
     m_columns_count = src.m_columns_count;
 
+    m_border_width = src.m_border_width;
+
     //Deep copy of the items map:
     remove_all_items();
     for(type_map_items::const_iterator iter = src.m_map_items.begin(); iter != src.m_map_items.end(); ++iter)
@@ -264,20 +268,13 @@ void LayoutGroup::remove_field(const Glib::ustring& table_name, const Glib::ustr
     sharedptr<LayoutItem_Field> field_item = sharedptr<LayoutItem_Field>::cast_dynamic(item);
     if(field_item)
     {
-      if(field_item->get_has_relationship_name()) //If it's related table.
+      if(field_item->get_table_used(Glib::ustring()) == table_name)
       {
-        sharedptr<const Relationship> relationship = field_item->get_relationship();
-        if(relationship)
+        if(field_item->get_name() == field_name)
         {
-          if(relationship->get_to_table() == table_name)
-          {
-            if(field_item->get_name() == field_name)
-            {
-              m_map_items.erase(iterItem);
-              iterItem = m_map_items.begin(); //Start again, because we changed the container.AddDel 
-              continue;
-            }
-          }
+          m_map_items.erase(iterItem);
+          iterItem = m_map_items.begin(); //Start again, because we changed the container.AddDel 
+          continue;
         }
       }
     }
@@ -396,6 +393,11 @@ Glib::ustring LayoutGroup::get_part_type_name() const
   return _("Group");
 }
 
+guint LayoutGroup::get_items_count() const
+{
+  return m_map_items.size();
+}
+
 /*
 void LayoutGroup::debug(guint level) const
 {
@@ -418,3 +420,13 @@ void LayoutGroup::debug(guint level) const
 }
 */
 
+double LayoutGroup::get_border_width() const
+{
+  return m_border_width;
+}
+
+void LayoutGroup::set_border_width(double border_width)
+{
+  m_border_width = border_width;
+}
+
