@@ -48,6 +48,8 @@ public:
 
   virtual Glib::ustring get_title_or_name() const;
 
+  Glib::ustring get_title_or_name_no_custom() const;
+
   sharedptr<const CustomTitle> get_title_custom() const;
   sharedptr<CustomTitle> get_title_custom();
   void set_title_custom(const sharedptr<CustomTitle>& title);
@@ -87,6 +89,10 @@ public:
 
   const FieldFormatting& get_formatting_used() const;
 
+  /** Compare the name, relationship, and related_relationship.
+   */
+  bool is_same_field(const sharedptr<const LayoutItem_Field>& field) const;
+
 protected:
 
   //This is just a cache, filled in by looking at the database structure:
