@@ -23,6 +23,8 @@
 
 #include "mode_data/dialog_layout.h"
 #include "data_structure/report.h"
+//#include "data_structure/layout/layoutitem.h"
+//#include "sharedptr.h"
 
 class Dialog_Layout_Report : public Dialog_Layout
 {
@@ -30,15 +32,13 @@ public:
   Dialog_Layout_Report(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_Layout_Report();
 
-  void set_report(const Glib::ustring& table_name, const Report& report);
-  Report get_report();
+  void set_report(const Glib::ustring& table_name, const sharedptr<const Report>& report);
+  sharedptr<Report> get_report();
 
   Glib::ustring get_original_report_name() const;
 
 protected:
 
-  typedef sharedptr<LayoutItem> type_item_ptr;
-
   class ModelColumnsGroups : public Gtk::TreeModel::ColumnRecord
   {
   public:
@@ -47,27 +47,26 @@ protected:
     { add(m_col_item); }
 
     //We use sharedptr<> so that the underlying LayoutItem will be destroyed automatically when the model is destroyed.
-    Gtk::TreeModelColumn< type_item_ptr > m_col_item;
+    Gtk::TreeModelColumn< sharedptr<LayoutItem> > m_col_item;
   };
 
   ModelColumnsGroups m_columns_parts;
 
   ModelColumnsGroups m_columns_available_parts;
 
-  virtual void add_group(const Gtk::TreeModel::iterator& parent, const LayoutGroup& group);
+  virtual void add_group(const Gtk::TreeModel::iterator& parent, const sharedptr<const LayoutGroup>& group);
 
-  void fill_group_children(LayoutGroup& group, const Gtk::TreeModel::iterator& iter);
-  LayoutGroup* fill_group(const Gtk::TreeModel::iterator& iter);
+  void fill_group_children(const sharedptr<LayoutGroup>& group, const Gtk::TreeModel::iterator& iter);
+  sharedptr<LayoutGroup> fill_group(const Gtk::TreeModel::iterator& iter);
 
   //Enable/disable buttons, depending on treeview selection:
   virtual void enable_buttons();
 
-  bool may_be_child_of(const LayoutItem& parent, const LayoutItem& suggested_child);
+  bool may_be_child_of(const sharedptr<const LayoutItem>& parent, const sharedptr<const LayoutItem>& suggested_child);
 
   virtual void save_to_document();
 
-  bool offer_relationship_list(Relationship& relationship);
-  bool offer_field_layout(LayoutItem_Field& field);
+  sharedptr<Relationship> offer_relationship_list();
   Gtk::TreeModel::iterator get_selected_group_parent() const;
 
   Gtk::TreeModel::iterator get_selected_available() const;
@@ -80,6 +79,7 @@ protected:
   virtual void on_button_delete();
 
   virtual void on_button_edit();
+  virtual void on_button_formatting();
 
   virtual void on_treeview_parts_selection_changed();
   virtual void on_treeview_available_parts_selection_changed();
@@ -97,6 +97,7 @@ protected:
   Gtk::Button* m_button_add;
   Gtk::Button* m_button_delete;
   Gtk::Button* m_button_edit;
+  Gtk::Button* m_button_formatting;
   Gtk::Label* m_label_table_name;
   Gtk::Entry* m_entry_name;
   Gtk::Entry* m_entry_title;
@@ -105,7 +106,7 @@ protected:
   Glib::RefPtr<Gtk::TreeStore> m_model_available_parts;
 
   Glib::ustring m_name_original;
-  Report m_report;
+  sharedptr<Report> m_report;
 };
 
 #endif //GLOM_DIALOG_LAYOUT_REPORT
