@@ -26,8 +26,10 @@
 #include "../data_structure/relationship.h"
 #include "../data_structure/field.h"
 #include "../data_structure/layout/layoutgroup.h"
+#include "../data_structure/layout/layoutitem_notebook.h"
 #include "../data_structure/layout/layoutitem_portal.h"
 #include "../data_structure/layout/layoutitem_button.h"
+#include "../data_structure/layout/layoutitem_text.h"
 #include "../data_structure/tableinfo.h"
 #include "../data_structure/groupinfo.h"
 #include "../data_structure/report.h"
@@ -142,6 +144,7 @@ public:
 
   typedef std::list< sharedptr<TranslatableItem> > type_list_translatables;
   type_list_translatables get_translatable_layout_items(const Glib::ustring& table_name);
+  type_list_translatables get_translatable_report_items(const Glib::ustring& table_name, const Glib::ustring& report_title);
 
   void fill_layout_field_details(const Glib::ustring& parent_table_name, const sharedptr<LayoutGroup>& layout_group) const;
   void fill_layout_field_details(const Glib::ustring& parent_table_name, type_mapLayoutGroupSequence& sequence) const;
@@ -240,7 +243,13 @@ public:
 
   virtual void emit_userlevel_changed();
 
-  static sharedptr<TableInfo> Document_Glom::create_table_system_preferences(type_vecFields& fields);
+  static sharedptr<TableInfo> create_table_system_preferences(type_vecFields& fields);
+  static bool get_relationship_is_system_properties(const sharedptr<const Relationship>& relationship);
+
+  static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, int value);
+  static void set_node_attribute_value_as_decimal_double(xmlpp::Element* node, const Glib::ustring& strAttributeName, double value);
+
+
 
 protected:
 
@@ -251,6 +260,9 @@ protected:
   void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, const sharedptr<LayoutGroup>& group);
   void save_before_layout_group(xmlpp::Element* node, const sharedptr<const LayoutGroup>& group);
 
+  void load_after_sort_by(const xmlpp::Element* node, const Glib::ustring table_name, LayoutItem_GroupBy::type_list_sort_fields& list_fields);
+  void save_before_sort_by(xmlpp::Element* node, const LayoutItem_GroupBy::type_list_sort_fields& list_fields);
+
   void load_after_layout_item_field(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<LayoutItem_Field>& item);
   void load_after_layout_item_field_formatting(const xmlpp::Element* element, FieldFormatting& format, Field::glom_field_type field_type, const Glib::ustring& table_name, const Glib::ustring& field_name);
 
@@ -270,8 +282,8 @@ protected:
   static bool get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
   static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true);
 
-  static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, int value);
   static guint get_node_attribute_value_as_decimal(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
+  static double get_node_attribute_value_as_decimal_double(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
 
   static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
   static Gnome::Gda::Value get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type);
