@@ -64,7 +64,7 @@ Box_Formatting::~Box_Formatting()
 {
 }
 
-void Box_Formatting::set_formatting(const FieldFormatting& format, const Glib::ustring& table_name, const Field& field)
+void Box_Formatting::set_formatting(const FieldFormatting& format, const Glib::ustring& table_name, const sharedptr<const Field>& field)
 {
   m_format = format;
   m_table_name = table_name;
@@ -84,30 +84,20 @@ void Box_Formatting::set_formatting(const FieldFormatting& format, const Glib::u
   //Choices:
   m_checkbutton_choices_restricted->set_active(format.get_choices_restricted());
 
+  const Document_Glom* document = get_document();
+
   //Fill the list of relationships:
-  m_combo_choices_relationship->clear_text();
+  const Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(m_table_name);
+  m_combo_choices_relationship->set_relationships(vecRelationships);
 
-  Document_Glom* document = get_document();
-  if(document)
-  {
-    Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(m_table_name);
-    for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); ++iter)
-    {
-      m_combo_choices_relationship->append_text(iter->get_name());
-    }
+  sharedptr<Relationship> choices_relationship;
+  Glib::ustring choices_field, choices_field_second;
+  format.get_choices(choices_relationship, choices_field, choices_field_second);
 
-    Glib::ustring choices_relationship, choices_field, choices_field_second;
-    format.get_choices(choices_relationship, choices_field, choices_field_second);
-
-    m_combo_choices_relationship->set_active_text(choices_relationship);
-    on_combo_choices_relationship_changed(); //Fill the combos so we can set their active items.
-    m_combo_choices_field->set_active_text(choices_field);
-    m_combo_choices_field_second->set_active_text(choices_field_second);
-  }
-  else
-  {
-    g_warning("Box_Formatting::set_formatting(): document not found.");
-  }
+  m_combo_choices_relationship->set_selected_relationship(choices_relationship);
+  on_combo_choices_relationship_changed(); //Fill the combos so we can set their active items.
+  m_combo_choices_field->set_active_text(choices_field);
+  m_combo_choices_field_second->set_active_text(choices_field_second);
 
   //Custom choices:
   m_adddel_choices_custom->remove_all();
@@ -115,7 +105,7 @@ void Box_Formatting::set_formatting(const FieldFormatting& format, const Glib::u
   for(FieldFormatting::type_list_values::const_iterator iter = list_choice_values.begin(); iter != list_choice_values.end(); ++iter)
   {
     //Display the value in the choices list as it would be displayed in the format:
-    Glib::ustring value_text = GlomConversions::get_text_for_gda_value(field.get_glom_type(), *iter, format.m_numeric_format);
+    Glib::ustring value_text = GlomConversions::get_text_for_gda_value(field->get_glom_type(), *iter, format.m_numeric_format);
     Gtk::TreeModel::iterator iter = m_adddel_choices_custom->add_item(value_text);
     m_adddel_choices_custom->set_value(iter, m_col_index_custom_choices, value_text);
   }
@@ -143,7 +133,8 @@ bool Box_Formatting::get_formatting(FieldFormatting& format) const
   //Choices:
   m_format.set_choices_restricted(m_checkbutton_choices_restricted->get_active());
 
-  m_format.set_choices(m_combo_choices_relationship->get_active_text(),
+  sharedptr<Relationship> choices_relationship = m_combo_choices_relationship->get_selected_relationship();
+  m_format.set_choices(choices_relationship,
     m_combo_choices_field->get_active_text(),
     m_combo_choices_field_second->get_active_text() );
 
@@ -158,7 +149,7 @@ bool Box_Formatting::get_formatting(FieldFormatting& format) const
       if(!text.empty())
       {
         bool success = false;
-        Gnome::Gda::Value value = GlomConversions::parse_value(m_field.get_glom_type(), text, m_format.m_numeric_format, success);
+        Gnome::Gda::Value value = GlomConversions::parse_value(m_field->get_glom_type(), text, m_format.m_numeric_format, success);
 
         if(success)
           list_choice_values.push_back(value);
@@ -176,22 +167,24 @@ bool Box_Formatting::get_formatting(FieldFormatting& format) const
 
 void Box_Formatting::on_combo_choices_relationship_changed()
 {
-  Glib::ustring relationship_name = m_combo_choices_relationship->get_active_text();
+  sharedptr<Relationship> relationship = m_combo_choices_relationship->get_selected_relationship();
 
   Document_Glom* pDocument = get_document();
   if(pDocument)
   {
     //Show the list of formats from this relationship:
-    Relationship relationship;
-    pDocument->get_relationship(m_table_name, relationship_name, relationship);
-    Document_Glom::type_vecFields vecFields = pDocument->get_table_fields(relationship.get_to_table());
-
-    m_combo_choices_field->clear_text();
-    m_combo_choices_field_second->clear_text();
-    for(Document_Glom::type_vecFields::const_iterator iter = vecFields.begin(); iter != vecFields.end(); ++iter)
+    if(relationship)
     {
-      m_combo_choices_field->append_text(iter->get_name());
-      m_combo_choices_field_second->append_text(iter->get_name());
+      Document_Glom::type_vecFields vecFields = pDocument->get_table_fields(relationship->get_to_table());
+
+      m_combo_choices_field->clear_text();
+      m_combo_choices_field_second->clear_text();
+      for(Document_Glom::type_vecFields::const_iterator iter = vecFields.begin(); iter != vecFields.end(); ++iter)
+      {
+        sharedptr<const Field> field = *iter;
+        m_combo_choices_field->append_text(field->get_name());
+        m_combo_choices_field_second->append_text(field->get_name());
+      }
     }
   }
 }
@@ -199,7 +192,7 @@ void Box_Formatting::on_combo_choices_relationship_changed()
 void Box_Formatting::enforce_constraints()
 {
   //Hide inappropriate UI:
-  const bool is_numeric = (m_field.get_glom_type() == Field::TYPE_NUMERIC);
+  const bool is_numeric = (m_field->get_glom_type() == Field::TYPE_NUMERIC);
   if(is_numeric)
     m_frame_numeric_format->show();
   else
@@ -207,7 +200,7 @@ void Box_Formatting::enforce_constraints()
     m_frame_numeric_format->hide();
   }
 
-  const bool is_text = (m_field.get_glom_type() == Field::TYPE_TEXT);
+  const bool is_text = (m_field->get_glom_type() == Field::TYPE_TEXT);
   if(is_text)
     m_frame_text_format->show();
   else
