@@ -27,6 +27,7 @@
 #include "../box_db.h"
 #include "../utility_widgets/combo_textglade.h"
 #include "../utility_widgets/comboentry_currency.h"
+#include "../combobox_relationship.h"
 
 class Box_Formatting
  : public Gtk::VBox,
@@ -41,7 +42,7 @@ public:
    * @param table_name The field's table.
    * @param The field that will have this formatting, so we know what formatting options to allow.
    */
-  virtual void set_formatting(const FieldFormatting& format, const Glib::ustring& table_name, const Field& field);
+  virtual void set_formatting(const FieldFormatting& format, const Glib::ustring& table_name, const sharedptr<const Field>& field);
   bool get_formatting(FieldFormatting& format) const;
 
 protected:
@@ -64,14 +65,14 @@ protected:
   Gtk::CheckButton* m_checkbutton_choices_restricted;
   AddDel_WithButtons* m_adddel_choices_custom;
   guint m_col_index_custom_choices;
-  Combo_TextGlade* m_combo_choices_relationship;
+  ComboBox_Relationship* m_combo_choices_relationship;
   Combo_TextGlade* m_combo_choices_field;
   Combo_TextGlade* m_combo_choices_field_second;
 
   mutable FieldFormatting m_format;
 
   Glib::ustring m_table_name;
-  Field m_field;
+  sharedptr<const Field> m_field;
 };
 
 #endif //GLOM_BOX_FORMATTING_H
