@@ -23,7 +23,6 @@
 
 #include "../box_db_table.h"
 #include "dialog_layout.h"
-#include "calcinprogress.h"
 
 
 /** Call init_db_details() to create the layout and fill it with data from the database.
@@ -84,11 +83,9 @@ protected:
   ///Fill the existing layout with data from the database.
   virtual bool fill_from_database(); //override.
 
-  virtual void do_lookups(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_changed, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
-  virtual void refresh_related_fields(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_changed, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
+  virtual void refresh_related_fields(const FieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
 
   virtual type_vecLayoutFields get_fields_to_show() const;
-  static Glib::ustring build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value);
   //virtual Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause);
   virtual bool get_related_record_exists(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value);
   virtual bool add_related_record_for_field(const sharedptr<const LayoutItem_Field>& layout_item_parent, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided);
@@ -100,44 +97,11 @@ protected:
   Document_Glom::type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout);
   void fill_layout_group_field_info(const sharedptr<LayoutGroup>& group, const Privileges& table_privs);
 
-  typedef std::pair< sharedptr<LayoutItem_Field>, sharedptr<Relationship> > type_pairFieldTrigger;
-  typedef std::list<type_pairFieldTrigger> type_list_lookups;
-
-  /** Get the fields whose values should be looked up when @a field_name changes, with
-   * the relationship used to lookup the value.
-   */
-  type_list_lookups get_lookup_fields(const Glib::ustring& field_name) const;
-
-  typedef std::map<Glib::ustring, CalcInProgress> type_field_calcs;
-
-  /** Get the fields whose values should be recalculated when @a field_name changes.
-   */
-  type_field_calcs get_calculated_fields(const Glib::ustring& field_name);
 
   /** Get the fields that are in related tables, via a relationship using @a field_name changes.
   */
   type_vecLayoutFields get_related_fields(const Glib::ustring& field_name) const;
 
-  /** Get the value of the @a source_field from the @a relationship, using the @a key_value.
-   */
-  Gnome::Gda::Value get_lookup_value(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value & key_value);
-
-  /** Calculate values for fields, set them in the database, and show them in the layout.
-   * @param field_changed The field that has changed, causing other fields to be recalculated because they use its value.
-   * @param primary_key The primary key field for this table.
-   * @param priamry_key_value: The primary key value for this record.
-   * @param first_calc_field: false if this is called recursively.
-   */
-  virtual void do_calculations(const sharedptr<const LayoutItem_Field>& field_changed, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value, bool first_calc_field = false);
-
-  /** Calculate a field value, show it and set it in the database.
-   * This will do the same for any dependent calculations.
-   */
-  void calculate_field(const sharedptr<const Field>& field, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value);
-
-  bool set_field_value_in_database(const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations = false);
-  bool set_field_value_in_database(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations = false);
-
   virtual bool record_delete(const Gnome::Gda::Value& primary_key_value);
   virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); //New record with all entered field values.
   Gnome::Gda::Value generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name);
@@ -151,11 +115,6 @@ protected:
   //Signal handlers:
   virtual void on_Button_Find(); //only used by _Find sub-classes. Should be MI.
 
-  typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_fields;
-  //TODO: Performance: This is massively inefficient:
-  type_map_fields get_record_field_values(const Gnome::Gda::Value& primary_key_value);
-
-
   static Glib::ustring xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt);
 
   Gtk::Button m_Button_Find; //only used by _Find sub-classes. Should be MI.
@@ -170,9 +129,6 @@ protected:
 
   type_vecFields m_TableFields; //A cache, so we don't have to repeatedly get them from the Document.
   type_vecLayoutFields m_FieldsShown; //And any extra keys needed by shown fields.
-
-
-  type_field_calcs m_FieldsCalculationInProgress; //Prevent circular calculations and recalculations.
 };
 
 #endif
