@@ -529,6 +529,9 @@ void DbAddDel::construct_specified_columns()
 
       record.add( *pModelColumn );
 
+      //if(iter->m_field->get_has_relationship_name())
+      //  std::cout << "  DEBUG: relationship=" << iter->m_field->get_relationship()->get_name() << std::endl;
+
       fields.push_back( glom_sharedptr_clone(iter->m_field) );
 
       i++;
@@ -883,13 +886,10 @@ DbAddDel::type_list_indexes DbAddDel::get_column_index(const sharedptr<const Lay
 
   type_list_indexes list_indexes;
 
-  const Glib::ustring field_name = layout_item->get_name();
-  const Glib::ustring relationship_name = layout_item->get_relationship_name();
-
   guint i = 0;
   for(type_ColumnTypes::const_iterator iter = m_ColumnTypes.begin(); iter != m_ColumnTypes.end(); ++iter)
   {
-    if( (iter->m_field->get_name() == field_name) && (iter->m_field->get_relationship_name() == relationship_name) )
+    if(iter->m_field->is_same_field(layout_item))
     {
       list_indexes.push_back(i);
     }
@@ -1088,7 +1088,8 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
     Gnome::Gda::Value value_old;
     row.get_value(tree_model_column_index, value_old);
 
-    bool bValueNew = !(value_old.get_bool());
+    const bool bValueOld = (value_old.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value_old.get_bool();
+    bool bValueNew = !bValueOld;
     Gnome::Gda::Value value_new;
     value_new.set(bValueNew);
     //Store the user's new value in the model:
@@ -1590,7 +1591,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
       {
         Gtk::CellRendererToggle* pDerived = dynamic_cast<Gtk::CellRendererToggle*>(renderer);
         if(pDerived)
-          pDerived->set_active( value.get_bool() ); 
+          pDerived->set_active( (value.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value.get_bool() ); 
 
         break;
       }
