@@ -17,10 +17,12 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 #include "flowtablewithfields.h"
 #include "datawidget.h"
 #include "buttonglom.h"
+#include "notebookglom.h"
+#include "labelglom.h"
 #include <gtkmm/checkbutton.h>
 #include "../data_structure/glomconversions.h"
 #include "../mode_data/box_data_list_related.h"
@@ -98,41 +100,32 @@ void FlowTableWithFields::add_layout_item_at_position(const sharedptr<LayoutItem
     sharedptr<LayoutItem_Portal> portal = sharedptr<LayoutItem_Portal>::cast_dynamic(item);
     if(portal)
     {
-      Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
-      if(pDocument)
-      {
-        sharedptr<Relationship> relationship = pDocument->get_relationship(m_table_name, portal->get_relationship_name());
-        if(relationship)
-        {
-          Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
-          add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
-
-          portal_box->init_db_details(portal); //Create the layout
-
-          portal_box->set_layout_item(portal, relationship->get_to_table());
-          portal_box->show();
-          add(*portal_box, true /* expand */);
-
-          m_portals.push_back(portal_box);
-          add_layoutwidgetbase(portal_box, add_before);
-
-          //Connect signals:
-          portal_box->signal_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_record_changed) );
-
-          portal_box->signal_user_requested_details().connect( sigc::bind( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_user_requested_details), portal_box));
-        }
-      }
+      add_layout_related_at_position(portal, add_before);
     }
     else
     {
-      sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::cast_dynamic(item);
-      if(group)
-        add_layout_group_at_position(group, add_before);
+      sharedptr<LayoutItem_Notebook> notebook = sharedptr<LayoutItem_Notebook>::cast_dynamic(item);
+      if(notebook)
+      {
+        add_layout_notebook_at_position(notebook, add_before);
+      }
       else
       {
-        sharedptr<LayoutItem_Button> layout_button = sharedptr<LayoutItem_Button>::cast_dynamic(item);
-        if(layout_button)
-          add_button_at_position(layout_button, m_table_name, add_before);
+        sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::cast_dynamic(item);
+        if(group)
+          add_layout_group_at_position(group, add_before);
+        else
+        {
+          sharedptr<LayoutItem_Button> layout_button = sharedptr<LayoutItem_Button>::cast_dynamic(item);
+          if(layout_button)
+            add_button_at_position(layout_button, m_table_name, add_before);
+          else
+          {
+            sharedptr<LayoutItem_Text> layout_textobject = sharedptr<LayoutItem_Text>::cast_dynamic(item);
+            if(layout_textobject)
+              add_textobject_at_position(layout_textobject, m_table_name, add_before);
+          }
+        }
       }
     }
   }
@@ -206,6 +199,138 @@ void FlowTableWithFields::add_layout_group_at_position(const sharedptr<LayoutGro
   }
 }
 
+Box_Data_List_Related* FlowTableWithFields::create_related(const sharedptr<LayoutItem_Portal>& portal, bool show_title)
+{
+  if(!portal)
+    return 0;
+
+  Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
+  if(pDocument)
+  {
+    sharedptr<Relationship> relationship = pDocument->get_relationship(m_table_name, portal->get_relationship_name());
+    if(relationship)
+    {
+      Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
+      add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
+
+      portal_box->init_db_details(portal, show_title); //Create the layout
+
+      portal_box->set_layout_item(portal, relationship->get_to_table());
+      portal_box->show();
+
+      m_portals.push_back(portal_box);
+
+      return portal_box;
+    }
+  }
+
+  return 0;
+}
+
+void FlowTableWithFields::add_layout_related_at_position(const sharedptr<LayoutItem_Portal>& portal, const type_list_layoutwidgets::iterator& add_before)
+{
+  Box_Data_List_Related* portal_box = create_related(portal);
+  if(portal_box)
+  {
+    add(*portal_box, true /* expand */);
+    add_layoutwidgetbase(portal_box, add_before);
+
+    //Connect signals:
+    portal_box->signal_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_record_changed) );
+
+    portal_box->signal_user_requested_details().connect( sigc::bind( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_user_requested_details), portal_box));
+  }
+}
+
+void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<LayoutItem_Notebook>& notebook, const type_list_layoutwidgets::iterator& add_before)
+{
+  if(!notebook)
+    return;
+
+  //Add the widget:
+  NotebookGlom* notebook_widget = Gtk::manage(new NotebookGlom());
+
+  notebook_widget->show();
+
+  for(LayoutGroup::type_map_items::iterator iter = notebook->m_map_items.begin(); iter != notebook->m_map_items.end(); ++iter)
+  {
+    sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::cast_dynamic(iter->second);
+    if(group)
+    {
+      Gtk::Label* tab_label = Gtk::manage(new Gtk::Label());
+      tab_label->show();
+
+      sharedptr<LayoutItem_Portal> portal = sharedptr<LayoutItem_Portal>::cast_dynamic(group);
+      if(portal)
+      {
+        const Glib::ustring tab_title = glom_get_sharedptr_title_or_name(portal->get_relationship());
+        tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+
+        //Add a Related Records list for this portal:
+        Box_Data_List_Related* portal_box = create_related(portal, false /* no label, because it's in the tab instead. */);
+        //portal_box->set_border_width(6); It has "padding" around the Alignment instead.
+        portal_box->show();
+        notebook_widget->append_page(*portal_box, *tab_label);
+
+        //add_layoutwidgetbase(portal_box, add_before);
+      }
+      else
+      {
+        const Glib::ustring tab_title = group->get_title_or_name();
+        tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+
+        //Add a FlowTable for this group:
+        FlowTableWithFields* flow_table = Gtk::manage( new FlowTableWithFields() );
+        add_view(flow_table); //Allow these sub-flowtables to access the document too.
+        flow_table->set_table(m_table_name);
+
+        flow_table->set_columns_count(group->m_columns_count);
+        flow_table->set_padding(6);
+        flow_table->show();
+
+        //This doesn't work (probably because we haven't implmented it in our custom container),
+        //so we put the flowtable in an alignment and give that a border instead.
+        //flow_table->set_border_width(6); //Put some space between the page child and the page edges.
+        Gtk::Alignment* alignment = Gtk::manage(new Gtk::Alignment());
+        alignment->set_border_width(6);
+        alignment->add(*flow_table);
+        alignment->show();
+
+        notebook_widget->append_page(*alignment, *tab_label);
+
+        //Add child items:
+        LayoutGroup::type_map_items items = group->get_items(); 
+        for(LayoutGroup::type_map_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+        {
+          sharedptr<LayoutItem> item = iter->second;
+          if(item)
+          {
+            flow_table->add_layout_item(item);
+          }
+        }
+
+        m_sub_flow_tables.push_back(flow_table);
+        flow_table->set_layout_item(group, m_table_name);
+        add_layoutwidgetbase(flow_table, add_before);
+
+        //Connect signal:
+        flow_table->signal_field_edited().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_edited) );
+        flow_table->signal_field_open_details_requested().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_open_details_requested) );
+        flow_table->signal_related_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_related_record_changed) );
+        flow_table->signal_requested_related_details().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_requested_related_details) );
+
+        flow_table->signal_script_button_clicked().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_script_button_clicked) );
+      }
+
+    }
+  }
+
+  add_layoutwidgetbase(notebook_widget, add_before);
+    //add_view(button); //So it can get the document.
+
+  add(*notebook_widget, true /* expand */);
+}
+
 /*
 void FlowTableWithFields::add_group(const Glib::ustring& group_name, const Glib::ustring& group_title, const type_map_field_sequence& fields)
 {
@@ -329,6 +454,37 @@ void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Butt
   add(*button);
 }
 
+void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_Text>& layoutitem_text, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
+{
+  //Add the widget:
+  Gtk::Alignment* alignment_label = Gtk::manage(new Gtk::Alignment());
+  alignment_label->set(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
+  alignment_label->show();
+
+  LabelGlom* label = Gtk::manage(new LabelGlom(layoutitem_text->get_text(), 0.0 /* xalign */, 0.5 /* yalign */)); //The alignment here seems to be necessary as well (or instead of) the parent Gtk::Alignment.
+  label->show();
+  alignment_label->add(*label);
+
+  add_layoutwidgetbase(label, add_before);
+  //add_view(button); //So it can get the document.
+
+  const Glib::ustring title = layoutitem_text->get_title();
+  if(title.empty())
+    add(*alignment_label);
+  else
+  {
+    Gtk::Alignment* alignment_title = Gtk::manage(new Gtk::Alignment());
+    alignment_title->set(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
+    alignment_title->show();
+
+    Gtk::Label* title_label = Gtk::manage(new Gtk::Label(title));
+    title_label->show();
+    alignment_title->add(*title_label);
+
+    add(*alignment_title, *alignment_label, true /* expand */);
+  }
+}
+
 void FlowTableWithFields::remove_field(const Glib::ustring& id)
 {
   for(type_listFields::iterator iter = m_listFields.begin(); iter != m_listFields.end(); ++iter)
@@ -451,11 +607,9 @@ FlowTableWithFields::type_list_const_widgets FlowTableWithFields::get_field(cons
 {
   type_list_const_widgets result;
 
-  const Glib::ustring layout_item_name = layout_item->get_name();
-  const Glib::ustring layout_item_relationship_name = layout_item->get_relationship_name();
   for(type_listFields::const_iterator iter = m_listFields.begin(); iter != m_listFields.end(); ++iter)
   {
-    if( (iter->m_field->get_name() == layout_item_name) && (iter->m_field->get_relationship_name() == layout_item_relationship_name) )
+    if( (iter->m_field->is_same_field(layout_item)) )
     {
       const Info& info = *iter;
       if(info.m_checkbutton)
@@ -488,11 +642,9 @@ FlowTableWithFields::type_list_widgets FlowTableWithFields::get_field(const shar
   //TODO: Avoid duplication
   type_list_widgets result;
 
-  const Glib::ustring layout_item_name = layout_item->get_name();
-  const Glib::ustring layout_item_relationship_name = layout_item->get_relationship_name();
   for(type_listFields::const_iterator iter = m_listFields.begin(); iter != m_listFields.end(); ++iter)
   {
-    if( (iter->m_field->get_name() == layout_item_name) && (iter->m_field->get_relationship_name() == layout_item_relationship_name) )
+    if( (iter->m_field->is_same_field(layout_item)) )
     {
       const Info& info = *iter;
       if(info.m_checkbutton)
@@ -759,6 +911,19 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
     layout_item->set_title(_("New Group"));
     add_layout_item_at_position(layout_item, iterAfter);
   }
+  else if(item_type == LayoutWidgetBase::TYPE_NOTEBOOK)
+  {
+    sharedptr<LayoutItem_Notebook> layout_item = sharedptr<LayoutItem_Notebook>::create();
+    layout_item->set_name(_("notebook"));
+
+    //Add an example tab, so that it shows up.
+    sharedptr<LayoutGroup> group_tab = sharedptr<LayoutGroup>::create();
+    group_tab->set_name(_("tab1"));
+    group_tab->set_title(_("Tab One"));
+    layout_item->add_item(group_tab);
+
+    add_layout_item_at_position(layout_item, iterAfter);
+  }
   else if(item_type == LayoutWidgetBase::TYPE_PORTAL)
   {
     try
@@ -795,6 +960,20 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
       std::cerr << ex.what() << std::endl;
     }
   }
+  else if(item_type == LayoutWidgetBase::TYPE_BUTTON)
+  {
+    sharedptr<LayoutItem_Button> layout_item = sharedptr<LayoutItem_Button>::create();
+    layout_item->set_name(_("button"));
+    layout_item->set_title(_("New Button"));
+    add_layout_item_at_position(layout_item, iterAfter);
+  }
+  else if(item_type == LayoutWidgetBase::TYPE_BUTTON)
+  {
+    sharedptr<LayoutItem_Text> layout_item = sharedptr<LayoutItem_Text>::create();
+    layout_item->set_name(_("text"));
+    layout_item->set_text(_("New Text"));
+    add_layout_item_at_position(layout_item, iterAfter);
+  }
 
   //TODO: Only if it has really changed:
   signal_layout_changed().emit(); //This should result in a complete re-layout.
@@ -807,6 +986,8 @@ void FlowTableWithFields::on_portal_record_changed(const Glib::ustring& relation
 
 void FlowTableWithFields::on_flowtable_related_record_changed(const Glib::ustring& relationship_name)
 {
+  //TODO_DoublyRelated
+
   //Forward it to the parent:
   signal_related_record_changed().emit(relationship_name);
 }
@@ -821,3 +1002,4 @@ void FlowTableWithFields::on_flowtable_requested_related_details(const Glib::ust
   //Forward it to the parent:
   signal_requested_related_details().emit(table_name, primary_key_value);
 }
+
