@@ -38,14 +38,14 @@ ImageGlom::ImageGlom()
 {
   setup_menu();
   setup_menu_usermode();
-  
+
   m_image.set_size_request(150, 150);
   m_image.show();
-  
+
   m_frame.set_shadow_type(Gtk::SHADOW_ETCHED_IN); //Without this, the image widget has no borders and is completely invisible when empty.
   m_frame.add(m_image);
   m_frame.show();
-  
+
   add(m_frame);
 }
 
@@ -58,7 +58,7 @@ bool ImageGlom::on_button_press_event(GdkEventButton *event)
 {
   GdkModifierType mods;
   gdk_window_get_pointer( Gtk::Widget::gobj()->window, 0, 0, &mods );
-      
+
   //Enable/Disable items.
   //We did this earlier, but get_application is more likely to work now:
   App_Glom* pApp = get_application();
@@ -91,7 +91,7 @@ bool ImageGlom::on_button_press_event(GdkEventButton *event)
         return true; //We handled this event.
       }
     }
-  
+
     //Single-click to select file:
     if(mods & GDK_BUTTON1_MASK)
     {
@@ -116,11 +116,11 @@ bool ImageGlom::get_has_original_data() const
 {
   return true; //TODO.
 }
-  
+
 void ImageGlom::set_value(const Gnome::Gda::Value& value)
-{ 
+{
   bool pixbuf_set = false;
-  
+
   if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
   {
     glong size = 0;
@@ -130,6 +130,7 @@ void ImageGlom::set_value(const Gnome::Gda::Value& value)
       //libgda does not currently properly unescape binary data,
       //so pData is actually a null terminated string, of escaped binary data.
       //This workaround should be removed when libgda is fixed:
+      //(It is fixed in libgd-2.0 but is unlikely to be fixed in libgda-1.2)
       size_t buffer_binary_length = 0;
       guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)pData /* must be null-terminated */, &buffer_binary_length); //freed by us later.
       if(buffer_binary)
@@ -141,16 +142,18 @@ void ImageGlom::set_value(const Gnome::Gda::Value& value)
         //{
         //  std::cout << " name=" << iter->get_name() << ", writable=" << iter->is_writable() << std::endl;
         //}
-        
+
         Glib::RefPtr<Gdk::PixbufLoader> refPixbufLoader;
-        
+
         // PixbufLoader::create() is broken in gtkmm before 2.6.something,
         // so let's do this in C so it works with all 2.6 versions:
         GError* error = 0;
         GdkPixbufLoader* loader = gdk_pixbuf_loader_new_with_type(GLOM_IMAGE_FORMAT, &error);
         if(!error)
           refPixbufLoader = Glib::wrap(loader);
-        
+        else
+          std::cerr << "ImageGlom::set_value(): Error while calling gdk_pixbuf_loader_new_with_type()." << std::endl;
+
         /*
         try
         {
@@ -163,44 +166,91 @@ void ImageGlom::set_value(const Gnome::Gda::Value& value)
           g_warning("PixbufLoader::create failed: %s",ex.what().c_str());
         }
         */
-        
+
         if(refPixbufLoader)
         {
           try
           {
             guint8* puiData = (guint8*)buffer_binary;
-            
+
             //g_warning("ImageGlom::set_value(): debug: from db: ");
             //for(int i = 0; i < 10; ++i)
             //  g_warning("%02X (%c), ", (guint8)puiData[i], (char)puiData[i]);
-              
+
             refPixbufLoader->write(puiData, (glong)buffer_binary_length);
+
             m_pixbuf_original = refPixbufLoader->get_pixbuf();
             m_image.set(m_pixbuf_original);
             pixbuf_set = true;
-            
+
             scale();
-            
+
             refPixbufLoader->close(); //This throws if write() threw, so it must be inside the try block.
           }
           catch(const Glib::Exception& ex)
           {
             g_warning("ImageGlom::set_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
           }
-          
+
           free(buffer_binary);
         }
       }
-            
+
       //TODO: load the image, using the mime type stored elsewhere.
       //pixbuf = Gdk::Pixbuf::create_from_data(
     }
-    
+
   }
-  
+
   if(!pixbuf_set)
   {
-    m_image.set(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_DIALOG);
+    /*
+    std::cout << "Debug: Setting MISSING_IMAGE" << std::endl;
+    
+    //Check that this stock icon size is really available,
+    //though it would be a distro error if it is not.
+    Glib::RefPtr<Gtk::Style> style = get_style();
+    if(style)
+    {
+      std::cout << "Debug: Setting MISSING_IMAGE 3" << std::endl;
+
+      const Gtk::IconSet iconset = style->lookup_icon_set(Gtk::Stock::MISSING_IMAGE);
+
+      std::cout << "Debug: Setting MISSING_IMAGE 4" << std::endl;
+
+      typedef std::vector<Gtk::IconSize> type_vecSizes;
+      type_vecSizes sizes = iconset.get_sizes();
+      type_vecSizes::iterator iterFind = std::find(sizes.begin(), sizes.end(), Gtk::ICON_SIZE_DIALOG);
+      if(iterFind != sizes.end())
+      {
+     */
+        m_image.set(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_DIALOG);
+     /*      
+      }
+      else
+      {
+        std::cerr << "Glom: The current theme does not seem to havae the Gtk::Stock::MISSING_IMAGE icon in size Gtk::ICON_SIZE_DIALOG" << std::endl;
+
+        if(!sizes.empty() && (sizes[0] > 0))
+        {
+          std::cerr << "  Using alternative stock icon size." << std::endl;
+          m_image.set(Gtk::Stock::MISSING_IMAGE, sizes[0]);
+        }
+        else
+        {
+          std::cerr << "  No alternative stock icon size available either, for this stock icon." << std::endl;
+          m_image.set("");
+        }
+      }
+    }
+    else
+    {
+      std::cerr << "Glom: No Gtk::Style available for this widget (yet), so not setting MISSING_IMAGE icon." << std::endl;
+      m_image.set("");
+    }
+    */
+
+    m_pixbuf_original = Glib::RefPtr<Gdk::Pixbuf>();
   }
 }
 
@@ -209,7 +259,7 @@ Gnome::Gda::Value ImageGlom::get_value() const
   //TODO: Return the data from the file that was just chosen.
   //Don't store the original here any longer than necessary,
   Gnome::Gda::Value result; //TODO: Initialize it as binary.
-  
+
   if(m_pixbuf_original)
   {
     try
@@ -220,46 +270,73 @@ Gnome::Gda::Value ImageGlom::get_value() const
       std::list<Glib::ustring> list_values;
       //list_keys.push_back("quality"); //For jpeg only.
       //list_values.push_back("95");
-      
+
       m_pixbuf_original->save_to_buffer(buffer, buffer_size, GLOM_IMAGE_FORMAT, list_keys, list_values); //Always store images as the standard format in the database.
-      
+
       //g_warning("ImageGlom::get_value(): debug: to db: ");
       //for(int i = 0; i < 10; ++i)
       //  g_warning("%02X (%c), ", (guint8)buffer[i], buffer[i]);
-          
-      result.set(buffer, buffer_size);
-      
+
+      //libgda currently assumes that this buffer is an already-escaped string.
+      //TODO: Just use the raw buffer when libgda (in libgda-2.0) has been fixed:
+      Glib::ustring binary_escaped = GlomConversions::get_escaped_binary_data((guint8*)buffer, buffer_size);
+      result.set(binary_escaped.c_str(), binary_escaped.size());
+
       g_free(buffer);
       buffer = 0;
     }
     catch(const Glib::Exception& /* ex */)
     {
-    
+
     }
   }
-  
+
+  return result;
+}
+
+bool ImageGlom::on_expose_event(GdkEventExpose* event)
+{
+  const bool result = Gtk::EventBox::on_expose_event(event);
+  scale();
   return result;
 }
 
 void ImageGlom::scale()
 {
   Glib::RefPtr<Gdk::Pixbuf> pixbuf = m_pixbuf_original;
-  
+
   if(pixbuf)
   {
     const Gtk::Allocation allocation = m_image.get_allocation();
     const int pixbuf_height = pixbuf->get_height();
     const int pixbuf_width = pixbuf->get_width();
-          
+
     if( (pixbuf_height > allocation.get_height()) ||
         (pixbuf_width > allocation.get_width()) )
     {
-      Glib::RefPtr<Gdk::Pixbuf> pixbuf_scaled = scale_keeping_ratio(pixbuf, allocation.get_height(), allocation.get_width());
-      m_image.set(pixbuf_scaled);
+      if(allocation.get_height() > 10 || allocation.get_width() > 10)
+      {
+        Glib::RefPtr<Gdk::Pixbuf> pixbuf_scaled = scale_keeping_ratio(pixbuf, allocation.get_height(), allocation.get_width());
+        if(!pixbuf_scaled)
+        {
+          std::cerr << "ImageGlom::scale(): scale_keeping_ratio() returned NULL pixbuf." << std::endl;
+        }
+        else 
+        {
+          //Don't set a new pixbuf if the dimenstions have not changed:
+          Glib::RefPtr<const Gdk::Pixbuf> pixbuf_in_image;
+
+          if(m_image.get_storage_type() == Gtk::IMAGE_PIXBUF) //Prevent warning.
+            pixbuf_in_image = m_image.get_pixbuf();
+
+          if( !pixbuf_in_image || (pixbuf_in_image->get_height() != pixbuf_scaled->get_height()) || (pixbuf_in_image->get_width() != pixbuf_scaled->get_width()) )
+            m_image.set(pixbuf_scaled);
+        }
+      }
     }
   }
-  else
-    g_warning("ImageGlom::scale(): attempt to scale a null pixbuf.");
+  //else
+  //  g_warning("ImageGlom::scale(): attempt to scale a null pixbuf.");
 }
 
 //static:
@@ -267,19 +344,19 @@ Glib::RefPtr<Gdk::Pixbuf> ImageGlom::scale_keeping_ratio(const Glib::RefPtr<Gdk:
 {
   if( (target_height == 0) || (target_width == 0) )
     return Glib::RefPtr<Gdk::Pixbuf>(); //This shouldn't happen anyway.
-    
+
   enum enum_scale_mode
   {
     SCALE_WIDTH,
     SCALE_HEIGHT,
     SCALE_NONE
   };
-    
+
   enum_scale_mode scale_mode = SCALE_NONE; //Start with either the width or height, and scale the other according to the ratio.
 
   const int pixbuf_height = pixbuf->get_height();
   const int pixbuf_width = pixbuf->get_width();
-  
+
   if(pixbuf_height > target_height)
   {
     if(pixbuf_width > target_width)
@@ -301,44 +378,43 @@ Glib::RefPtr<Gdk::Pixbuf> ImageGlom::scale_keeping_ratio(const Glib::RefPtr<Gdk:
     //Only the height is bigger:
     scale_mode = SCALE_WIDTH;
   }
-  
+
   if(scale_mode == SCALE_NONE)
     return pixbuf;
   else if(scale_mode == SCALE_HEIGHT)
   {
-    float ratio = (float)target_height / (float)pixbuf_height; 
+    const float ratio = (float)target_height / (float)pixbuf_height; 
     target_width = (int)((float)pixbuf_width * ratio);
   }
   else if(scale_mode == SCALE_WIDTH)
   {
-    float ratio = (float)target_width / (float) pixbuf_width;
+    const float ratio = (float)target_width / (float) pixbuf_width;
     target_height = (int)((float)pixbuf_height * ratio);
   }
-  
+
  if( (target_height == 0) || (target_width == 0) )
  {
-   g_warning("ImageGlom::scale_keeping_ratio(): calculated dimension is zero: target_width=%d, target_height=%d", target_width, target_height); 
-   return Glib::RefPtr<Gdk::Pixbuf>(); //This shouldn't happen anyway.
+   return Glib::RefPtr<Gdk::Pixbuf>(); //This shouldn't happen anyway. It seems to happen sometimes though, when ratio is very small.
  }
-    
+
   return pixbuf->scale_simple(target_width, target_height, Gdk::INTERP_NEAREST);
 }
 
 void ImageGlom::on_menupopup_activate_select_file()
 {
   Gtk::FileChooserDialog dialog(_("Choose image"), Gtk::FILE_CHOOSER_ACTION_OPEN);
-  
+
   //Get image formats only:
   Gtk::FileFilter filter;
   filter.set_name(_("Images"));
   filter.add_pixbuf_formats();
   dialog.add_filter(filter);
-  
+
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   dialog.add_button(_("Select"), Gtk::RESPONSE_OK);
   int response = dialog.run();
   dialog.hide();
-  
+
   if(response != Gtk::RESPONSE_CANCEL)
   {
     const std::string filepath = dialog.get_filename();
@@ -362,10 +438,10 @@ void ImageGlom::on_menupopup_activate_select_file()
       {
         App_Glom* pApp = get_application();
         if(pApp)
-          Frame_Glom::show_ok_dialog(_("Image loading failed"), _("The image file could not be opened:\n") + ex.what(), *pApp);
+          Frame_Glom::show_ok_dialog(_("Image loading failed"), _("The image file could not be opened:\n") + ex.what(), *pApp, Gtk::MESSAGE_ERROR);
       }
     }
-  }        
+  }
 }
 
 void ImageGlom::on_clipboard_get(Gtk::SelectionData& selection_data, guint /* info */)
@@ -393,18 +469,22 @@ void ImageGlom::on_clipboard_clear()
 
 void ImageGlom::on_menupopup_activate_copy()
 {
-  //When copy is used, store it here until it is pasted.
-  m_pixbuf_clipboard = m_pixbuf_original->copy(); //TODO: Get it from the DB, when we stop storing the original here instead of just the preview.
-  
+  if(m_pixbuf_original)
+  {
+    //When copy is used, store it here until it is pasted.
+    m_pixbuf_clipboard = m_pixbuf_original->copy(); //TODO: Get it from the DB, when we stop storing the original here instead of just the preview.
+  }
+  else
+    m_pixbuf_clipboard.clear();
+
   Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
 
   //Targets:
   std::list<Gtk::TargetEntry> listTargets;
 
   listTargets.push_back( Gtk::TargetEntry(GLOM_IMAGE_FORMAT_MIME_TYPE) );
-  
+
   refClipboard->set( listTargets, sigc::mem_fun(*this, &ImageGlom::on_clipboard_get), sigc::mem_fun(*this, &ImageGlom::on_clipboard_clear) );
- 
 }
 
 void ImageGlom::on_clipboard_received_image(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
@@ -412,7 +492,7 @@ void ImageGlom::on_clipboard_received_image(const Glib::RefPtr<Gdk::Pixbuf>& pix
   if(pixbuf)
   {
     m_pixbuf_original = pixbuf;
-    
+
     m_image.set(m_pixbuf_original); //Load the image.
     scale();
     signal_edited().emit();
@@ -425,25 +505,27 @@ void ImageGlom::on_menupopup_activate_paste()
   //Tell the clipboard to call our method when it is ready:
   Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
 
-  refClipboard->request_image( sigc::mem_fun(*this, &ImageGlom::on_clipboard_received_image) );
+  if(refClipboard)
+    refClipboard->request_image( sigc::mem_fun(*this, &ImageGlom::on_clipboard_received_image) );
 }
 
 void ImageGlom::on_menupopup_activate_clear()
 {
   m_pixbuf_original.clear();
   m_image.set(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_DIALOG);
+  signal_edited().emit();
 }
 
 void ImageGlom::setup_menu_usermode()
 {
   m_refActionGroup_UserModePopup = Gtk::ActionGroup::create();
-  
+
   m_refActionGroup_UserModePopup->add(Gtk::Action::create("ContextMenu_UserMode", "Context Menu") );
   m_refActionSelectFile =  Gtk::Action::create("ContextSelectFile", Gtk::Stock::EDIT, _("Choose File"));
   m_refActionCopy = Gtk::Action::create("ContextCopy", Gtk::Stock::COPY);
   m_refActionPaste = Gtk::Action::create("ContextPaste", Gtk::Stock::PASTE);
   m_refActionClear = Gtk::Action::create("ContextClear", Gtk::Stock::CLEAR);
-  
+
   m_refActionGroup_UserModePopup->add(m_refActionSelectFile,
     sigc::mem_fun(*this, &ImageGlom::on_menupopup_activate_select_file) );
 
@@ -452,7 +534,7 @@ void ImageGlom::setup_menu_usermode()
 
   m_refActionGroup_UserModePopup->add(m_refActionPaste,
     sigc::mem_fun(*this, &ImageGlom::on_menupopup_activate_paste) );
-    
+
   m_refActionGroup_UserModePopup->add(m_refActionClear,
     sigc::mem_fun(*this, &ImageGlom::on_menupopup_activate_clear) );
 
